from odoo import api, fields, models, _
from odoo.exceptions import UserError, ValidationError

import logging
import re
from lxml import etree

_logger = logging.getLogger(__name__)

class CustomRepairInherit(models.Model):
    _inherit = 'repair.order'

    product_id = fields.Many2one(
        'product.product',
        string='Pieza a reparar',
        readonly=True, required=False, states={'order': [('readonly', False)]}
    )
    
    product_template_id = fields.Many2one(
        'product.template', 
        string='Pieza a reparar',
        domain="[('categ_id.complete_name', '=', 'ALMACÉN / REPARACIONES')]",
    )

    new_product_template_id = fields.Char(
        string='Nuevo N Identificador',
    )

    new_product_template_id_visible = fields.Boolean(default=False)

    autogenerated_product_template_id = fields.Many2one(
        'product.template', 
        string='Autogenerated Id number',
        domain="[('categ_id.complete_name', '=', 'ALMACÉN / REPARACIONES')]",    
    )

    product_type = fields.Many2one(
        'product.category', 
        string='Product type', 
        readonly=True
    )
    product_qty = fields.Float(
        'Product Quantity',
        default=1.0, digits='Product Unit of Measure',
        readonly=True, required=True, states={'order': [('readonly', False)]})
    product_uom = fields.Many2one(
        'uom.uom', 'Product Unit of Measure',
        readonly=True, required=False, states={'order': [('readonly', False)]}, domain="[('category_id', '=', product_uom_category_id)]")
    
    state = fields.Selection([
        ('draft', 'Quotation'),
        ('confirmed', 'Confirmed'),
        ('ready', 'Ready to Repair'),
        ('under_repair', 'Under Repair'),
        ('done', 'Repaired'),
        ('cancel', 'Cancelled'),
        ('order', 'Order'),
        ('analysis', 'Analysis'),
        ('repair', 'Repair'),
        ('verification', 'Verification'),
        ('2binvoiced', 'Billing')
        ], string='Status',
        copy=False, default='order', readonly=True, tracking=True
    )
    
    user_id = fields.Many2one('res.users', string="Responsible", default=lambda self: self.env.user, check_company=True)

    def _default_location_id(self):
        location = self.env['stock.location'].search([('complete_name', '=', 'AT/Stock')], limit=1)
        return location.id if location else False

    location_id = fields.Many2one(
        'stock.location', 'Location',
        index=True, readonly=True, required=True, check_company=True,
        help="This is the location where the product to repair is located.",
        states={'order': [('readonly', False)], 'confirmed': [('readonly', True)]},
        default=_default_location_id
    )
    
    invoice_method = fields.Selection([
        ("none", "No Invoice"),
        ("b4repair", "Before Repair"),
        ("after_repair", "After Repair")], string="Invoice Method",
        default='none', index=True, readonly=True, required=True,
        states={'order': [('readonly', False)]}
    )
    
    product_number_id = fields.Many2one(related='pricelist_id.currency_id')

    timesheet_lines = fields.One2many(
        'repair.fee', 'repair_id', 'Operations',
        copy=True, readonly=False)
    
    timesheet_ids = fields.One2many(
        'account.analytic.line',  # El modelo de líneas de tiempo
        'repair_id',  # El campo relacional que debes crear en `account.analytic.line`
        string="Hojas de Tiempo"
    )

    repair_stage = fields.Selection([
        ('desarmado', 'Desarmado'),
        ('caract', 'Toma de Características'),
        ('sacado_alambre', 'Sacado de Alambre'),
        ('estator', 'Limpieza, Aislación y Bobinado Estator'),
        ('barnizado', 'Barnizado y Horneado'),
        ('limpieza_pintura', 'Limpieza y Pintura'),
        ('armado', 'Armado')
    ], 
    string='Etapa de Reparación', default='desarmado', 
    group_expand='_group_expand_states', track_visibility='always')

    repair_type = fields.Selection([
        ('mantenimiento', 'Bobinado y Reparación'),
        ('mantRep', 'Reparación y Mantenimiento'),
        ('internal', 'Servicio Interno'),
        ('others', 'Otros Servicios'),
        ('warranty', 'Garantía'),
        ('reingreso', 'Reingreso'),
        ('low', 'Baja'),
    ], 
    string='Tipo de Reparación')

    timesheet_hour_cost = fields.Many2one(
        'product.template', string='Costo de horas', domain=[('type', '=', 'service'), ('service_policy', '=', 'delivered_timesheet')],
        default=lambda self: self.env['product.template'].search([('name', 'ilike', 'timesheet')], limit=1).id)
    
    is_operator = fields.Boolean(compute='_compute_is_operator')

    is_chief_operator = fields.Boolean(compute='_compute_is_chief_operator')

    picking_ids = fields.Many2many('stock.picking', string='Picking associated to this manufacturing order')
    delivery_count = fields.Integer(compute='_compute_picking_ids', string='Delivery Orders')
    repair_finished_bool = fields.Boolean(default=False, store=True)
    
    @api.depends('picking_ids')
    def _compute_picking_ids(self):
        for order in self:
            order.delivery_count = len(order.picking_ids)

    def _compute_is_operator(self):
        for rec in self:
            rec.is_operator = self.env.user.has_group('custom_repair.group_operario_reparacion')
    
    def _compute_is_chief_operator(self):
        for rec in self:
            rec.is_chief_operator = self.env.user.has_group('custom_repair.group_jefe_reparacion')

    @api.model
    def create(self, vals):
        if self.is_operator:
            raise UserError(_("No tiene permisos para crear una orden de reparación."))
        
        return super(CustomRepairInherit, self).create(vals)

    def write(self, vals):
        if self.is_operator and not self.is_chief_operator and 'repair_stage' in vals and vals['repair_stage'] != self.repair_stage:
            raise UserError(_("No tiene permisos para cambiar el estado de reparación."))
        #if self.is_operator and ['repair_stage', 'description', 'description'] in vals and vals['repair_stage'] != self.repair_stage:
        #    raise UserError(_("No tiene permisos para cambiar el estado de reparación."))
        if 'repair_stage' in vals and vals['repair_stage'] != self.repair_stage:
            self.repair_finished_bool = False
        if self.is_operator and not self.is_chief_operator and self.state == 'repair' and self.user_id != self.env.user:
            raise UserError(_("No puede modificar una orden de reparación no asignada."))

        return super(CustomRepairInherit, self).write(vals)
    
    @api.model
    def fields_view_get(self, view_id=None, view_type='form',
                        toolbar=False, submenu=False):
        ret_val = super(CustomRepairInherit, self).fields_view_get(
            view_id=view_id, view_type=view_type,
            toolbar=toolbar, submenu=submenu)
        
        if self.env.user.has_group('custom_repair.group_operario_reparacion') and view_type in ['form', 'tree', 'kanban']:
            doc = etree.XML(ret_val['arch'])
            for node in doc.xpath("//form|//tree|//kanban"):
                node.set('create','false')
            ret_val['arch'] = etree.tostring(doc)
        return ret_val

    def action_order(self):
        self.ensure_one()
        if not self.lot_id:
            raise UserError(_("Por favor completar el numero de serie de la orden."))
        if not self.product_id:
            raise UserError(_("Please complete product before confirm the order."))
        if self.filtered(lambda repair: any(op.product_uom_qty < 0 for op in repair.operations)):
            raise UserError(_("You can not enter negative quantities."))
        if self.product_id.type == 'consu':
            return self.action_repair_order_confirm()
        precision = self.env['decimal.precision'].precision_get('Product Unit of Measure')
        available_qty_owner = sum(self.env['stock.quant'].search([
            ('product_id', '=', self.product_id.id),
            ('location_id', '=', self.location_id.id),
            ('lot_id', '=', self.lot_id.id),
            ('owner_id', '=', self.partner_id.id),
        ]).mapped('quantity'))
        available_qty_noown = sum(self.env['stock.quant'].search([
            ('product_id', '=', self.product_id.id),
            ('location_id', '=', self.location_id.id),
            ('lot_id', '=', self.lot_id.id),
            ('owner_id', '=', False),
        ]).mapped('quantity'))
        repair_qty = self.product_uom._compute_quantity(self.product_qty, self.product_id.uom_id)
        
        return self.action_repair_order_confirm()
    
    def action_repair_order_confirm(self):
        """ Repair order state is set to 'To be invoiced' when invoice method
        is 'Before repair' else state becomes 'Confirmed'.
        @param *arg: Arguments
        @return: True
        """
        if not self.product_id:
            raise UserError(_("Please complete product before confirm the order."))
        if self.filtered(lambda repair: repair.state != 'order'):
            raise UserError(_("Only order repairs can be confirmed."))
        self._check_company()
        self.operations._check_company()
        self.fees_lines._check_company()
        before_repair = self.filtered(lambda repair: repair.invoice_method == 'b4repair')
        before_repair.write({'state': '2binvoiced'})
        to_confirm = self - before_repair
        to_confirm_operations = to_confirm.mapped('operations')
        to_confirm_operations.write({'state': 'analysis'})
        to_confirm.write({'state': 'analysis'})
        self.action_generate_stock_move_in()
        return True

    def action_repair_cancel_order(self):
        if self.filtered(lambda repair: repair.state != 'cancel'):
            raise UserError(_("Repair must be canceled in order to reset it to draft."))
        self.mapped('operations').write({'state': 'order'})
        return self.write({'state': 'order', 'invoice_id': False})
    
    def action_repair_cancel(self):
        # Elimino el producto autogenerado si existe
        '''
        if self.product_template_id.autogenerated:
            self.product_template_id.unlink()
            # Mostrar notificación
            self.env.user.notify_info(message="El producto autogenerado ha sido eliminado.")
        '''
        order_state = self.state
        super(CustomRepairInherit, self).action_repair_cancel()
        if order_state != 'order':
            self.action_generate_stock_move_out()

        return True

    def action_repair_analysis_confirm(self):
        if self.filtered(lambda repair: repair.state != 'analysis'):
            raise UserError(_("Only analysis repairs can be confirmed."))
        self._check_company()
        self.operations._check_company()
        self.fees_lines._check_company()
        before_repair = self.filtered(lambda repair: repair.invoice_method == 'b4repair')
        before_repair.write({'state': '2binvoiced'})
        to_confirm = self - before_repair
        to_confirm_operations = to_confirm.mapped('operations')
        to_confirm_operations.write({'state': 'repair'})
        to_confirm.write({'state': 'repair'})
        return True
    
    def action_repair_confirm(self):
        if 'T' in self.tag_ids.mapped('name'):
            raise UserError(_("No se puede pasar a verificación una orden de reparación que se encuentra en torneria."))
        if self.filtered(lambda repair: repair.state != 'repair'):
            raise UserError(_("Only analysis repairs can be confirmed."))
        self._check_company()
        self.operations._check_company()
        self.fees_lines._check_company()
        before_repair = self.filtered(lambda repair: repair.invoice_method == 'b4repair')
        before_repair.write({'state': '2binvoiced'})
        to_confirm = self - before_repair
        to_confirm_operations = to_confirm.mapped('operations')
        to_confirm_operations.write({'state': 'verification'})
        to_confirm.write({'state': 'verification'})
        '''
        for operation in self.operations:
            operation.location_id = self.location_id
        '''
        if self.is_chief_operator:
            return {
                'type': 'ir.actions.act_window',
                'name': 'Órdenes en reparación',
                'res_model': 'repair.order',
                'view_mode': 'kanban',
                'view_id': self.env.ref('custom_repair.view_custom_repair_order_kanban').id,
                'domain': [('state', '=', 'repair')],
                'context': {},
                'target': 'main',
            }
        else:
            return True

    def action_repair_verification_confirm(self):
        if self.filtered(lambda repair: repair.state != 'verification'):
            raise UserError(_("Repair must be under repair in order to end reparation."))
        self._check_product_tracking()
        if not self.timesheet_hour_cost:
            raise UserError(_("Please set a timesheet cost."))

        service_product = self.env['product.template'].search([
                ('type', '=', 'service'),
                ('invoice_service_product', '=', True)], limit=1)
        service_product_id = self.env['product.product'].search([
                ('product_tmpl_id', '=', service_product.id)
        ])
        if not service_product:
            raise UserError(_("Please set a service product for the repair."))
        service_hours_cost = 0
        for timesheet_id in self.timesheet_ids:
            if self.timesheet_hour_cost.currency_id != self.pricelist_id.currency_id:
                hours_price = self.timesheet_hour_cost.currency_id._convert(self.timesheet_hour_cost.list_price, self.pricelist_id.currency_id, self.company_id, timesheet_id.date)
            else:
                hours_price = self.timesheet_hour_cost.list_price
            service_hours_cost += timesheet_id.unit_amount * hours_price

        for fee in self.fees_lines:
            if fee.product_id.categ_id.name == 'ADELANTOS':
                fee.price_unit = -fee.price_unit
                fee.price_subtotal = -fee.price_subtotal

        parts_cost = 0
        for operation in self.operations:
            parts_cost += operation.price_subtotal
            operation.price_unit = 0

        values = {
            'repair_id': self.id,
            'name': service_product.name,
            'product_id': service_product_id.id,
            'price_unit': service_hours_cost + parts_cost,
            'currency_id': service_product.currency_id.id,
            'product_uom_qty': 1,
            'product_uom': service_product.uom_id.id,
            'company_id': service_product.company_id.id,
            'tax_id': service_product.taxes_id.ids,
        }

        self.fees_lines.sudo().create(values)

        for repair in self:
            repair.write({'repaired': True})
            repair.write({'invoice_method': 'after_repair'})
            vals = {'state': 'done'}
            vals['move_id'] = repair.action_repair_done().get(repair.id)
            if not repair.invoice_id and repair.invoice_method == 'after_repair':
                vals['state'] = '2binvoiced'
            repair.write(vals)
        return True
    
    def action_repair_invoice_create(self):
        res = super(CustomRepairInherit, self).action_repair_invoice_create()
        self.action_generate_stock_move_out()

        return res

    
    @api.depends('product_template_id')
    def _compute_product_details(self):
        for record in self:
            if record.product_template_id:
                record.product_id = record.product_template_id.product_variant_id
                record.product_type = record.product_template_id.categ_id
            else:
                record.product_id = False
                record.product_type = False

    @api.onchange('product_template_id')
    def _onchange_product_template_id(self):
        """Autocompletar product_type basado en la categoría del producto seleccionado"""
        if self.product_template_id:
            self.product_id = self.product_template_id.product_variant_id
            self.product_type = self.product_template_id.categ_id
        else:
            self.product_type = False
            self.product_id = False

    '''
    @api.onchange('repair_stage')
    def _onchange_repair_stage(self):
        if 'T' in self.tag_ids.mapped('name'):
            raise UserError(_("No se puede mover la orden de reparación de la etapa actual porque se encuentra en torneria."))
    '''
    
    @api.onchange('company_id')
    def _onchange_company_id(self):
        super(CustomRepairInherit, self)._onchange_company_id()
        if self.state == 'order' and not self.product_template_id:
            #self.generate_new_product()
            print()
    
    
    @api.model
    def _compute_product_template_id(self):
        # Buscar el número más grande de product_template_id y devolver el siguiente número en el formato "IMO00000001"
        prod_template = self.env['product.template']
        #categ_id = self.env['product.category'].search([('complete_name', '=', 'ALMACÉN / REPARACIONES')], limit=1)
        last_product_template = prod_template.search([('categ_id.complete_name', '=', 'ALMACÉN / REPARACIONES')], order='id desc', limit=1)
        if last_product_template:
            last_code = last_product_template.default_code
            if last_code and last_code.startswith('IMO'):
                last_number = int(last_code[3:])
                new_number = last_number + 1
                default_code = 'IMO{:08d}'.format(new_number)
        else:    
            default_code = 'IMO00000001'

        return default_code

    def _group_expand_states(self, states, domain, order):
        return [key for key, val in type(self).repair_stage.selection]
    
    def create_new_product_with_name(self, name):     
        categ_id = self.env['product.category'].search([('complete_name', '=', 'ALMACÉN / REPARACIONES')], limit=1)
        new_product_template = self.env['product.template'].create({
            'name': name,
            'default_code': name,
            'categ_id': categ_id.id,
            'qty_available': 1,
            'detailed_type': 'product',
            'type': 'product',
        })
        self.product_template_id = new_product_template
        self.product_id = new_product_template.product_variant_id
        self.product_type = new_product_template.categ_id   
        
        return new_product_template

    def generate_new_product(self):     
        default_code = self._compute_product_template_id()
        categ_id = self.env['product.category'].search([('complete_name', '=', 'ALMACÉN / REPARACIONES')], limit=1)
        new_product_template = self.env['product.template'].new({
            'name': default_code,
            'default_code': default_code,
            'categ_id': categ_id.id,
            'qty_available': 1,
            'autogenerated': True,
            'detailed_type': 'product',
            'type': 'product',
        })
        self.product_template_id = new_product_template
        self.product_id = new_product_template.product_variant_id
        self.product_type = new_product_template.categ_id   
        
        return new_product_template

    def create_new_product(self):     
        if not self.autogenerated_product_template_id:  
            default_code = self._compute_product_template_id()
            categ_id = self.env['product.category'].search([('complete_name', '=', 'ALMACÉN / REPARACIONES')], limit=1)
            new_product_template = self.env['product.template'].create({
                'name': default_code,
                'default_code': default_code,
                'categ_id': categ_id.id,
                'qty_available': 1,
                'autogenerated': True,
                'detailed_type': 'product',
                'type': 'product',
            })
            self.autogenerated_product_template_id = new_product_template
            self.product_template_id = new_product_template
            self.product_id = new_product_template.product_variant_id
            self.product_type = new_product_template.categ_id   
        
            return new_product_template
        
        else:
            raise UserError(_('Ya se autogeneró un nuevo producto para esta orden de reparación:\n Producto: %s.') % (self.autogenerated_product_template_id.name))            
    
    def action_create_new_product(self):
        # Create a new id number
        #self.create_new_product()
        self.action_generate_serial()

    def action_new_product_visible(self):
        if self.new_product_template_id_visible:
            self.new_product_template_id_visible = False
            self.new_product_template_id = ''
        else:
            self.new_product_template_id_visible = True

    def action_create_new_product_with_name(self):
        # Crear un nuevo product.template con cierto nombre
        name = self.new_product_template_id
        if not re.match(r'^IMO\d{8}$', name):
            raise ValidationError(_("Validation Error: The name must match the pattern 'IMO########'."))
        if self.env['product.template'].search([('default_code', '=', name)]):
            raise ValidationError(_("Validation Error: The name must be unique."))
        self.create_new_product_with_name(name)
        self.new_product_template_id_visible = False
        self.new_product_template_id = ''
    
    def action_generate_serial(self):
        self.ensure_one()
        if self.product_id.tracking == 'none':
            return
        spl = self.env['stock.production.lot']
        name = spl._get_new_serial(self.company_id, self.product_id)
        self.lot_id = self.env['stock.production.lot'].create({
            'product_id': self.product_id.id,
            'company_id': self.company_id.id,
            'name': name,
        })

    def action_generate_stock_move_in(self):
        self.ensure_one()
        sm = self.env['stock.move']
        sml = self.env['stock.move.line']
        origin_location = self.env['stock.location'].search([('complete_name', '=', 'Virtual Locations/Reparaciones')], limit=1)
        if not origin_location:
            raise UserError(_("Por favor crear la ubicación de ajuste de inventario con nombre 'Virtual Locations/Reparaciones'"))
        picking_type = self.env['stock.picking.type'].search([('default_location_dest_id', '=', self.location_id.id), ('code', '=', 'incoming'), ('use_existing_lots', '=', False)], limit=1)
        new_sm = sm.create({
            'name': 'Ingreso de producto a reparar ' + self.lot_id.name,
            'picking_type_id': picking_type.id,
            'product_id': self.product_id.id,
            'product_uom_qty': self.product_qty,
            'product_uom': self.product_uom.id,	
            'location_id': origin_location.id,	
            'location_dest_id': self.location_id.id,
            'state': 'done',
        })
        new_sml = sml.create({
            'move_id': new_sm.id,
            'product_id': self.product_id.id,
            'qty_done': self.product_qty,
            'lot_id': self.lot_id.id,
            'product_uom_id': self.product_uom.id,	
            'location_id': origin_location.id,	
            'location_dest_id': self.location_id.id,
            'state': 'done',
        })

        return
    
    def action_generate_stock_move_out(self):
        self.ensure_one()
        sm = self.env['stock.move']
        sml = self.env['stock.move.line']
        sp = self.env['stock.picking']
        dest_location = self.env['stock.location'].search([('complete_name', '=', 'Virtual Locations/Reparaciones')], limit=1)
        if not dest_location:
            raise UserError(_("Por favor crear la ubicación de ajuste de inventario con nombre 'Virtual Locations/Reparaciones'"))
        picking_type = self.env['stock.picking.type'].search([('default_location_src_id', '=', self.location_id.id), ('code', '=', 'outgoing'), ('use_create_lots', '=', False)], limit=1)
        picking_out = sp.create({
            'origin': self.name,
            'partner_id': self.partner_id.id,
            'picking_type_id': picking_type.id,	
            'location_id': self.location_id.id,	
            'location_dest_id': dest_location.id,
            'immediate_transfer': True,
            'state': 'assigned',
        })
        new_sm = sm.create({
            'name': 'Ingreso de producto a reparar ' + self.lot_id.name,
            'picking_type_id': picking_type.id,
            'product_id': self.product_id.id,
            'product_uom_qty': self.product_qty,
            'product_uom': self.product_uom.id,	
            'location_id': self.location_id.id,	
            'location_dest_id': dest_location.id,
            'state': 'assigned',
        })
        new_sml = sml.create({
            'move_id': new_sm.id,
            'product_id': self.product_id.id,
            'qty_done': self.product_qty,
            'lot_id': self.lot_id.id,
            'product_uom_id': self.product_uom.id,	
            'location_id': self.location_id.id,	
            'location_dest_id': dest_location.id,
            'state': 'assigned',
        })
        picking_out.move_lines = [(4, new_sm.id)]
        picking_out.move_line_ids = [(4, new_sml.id)]
        #picking_out.button_validate()
        self.picking_ids = [(4, picking_out.id)]

        return

        
    def action_send_mail(self):
        self.ensure_one()
        template_id = self.env.ref('custom_repair.mail_template_custom_repair_quotation').id
        ctx = {
            'default_model': 'repair.order',
            'default_res_id': self.id,
            'default_use_template': bool(template_id),
            'default_template_id': template_id,
            'default_composition_mode': 'comment',
            'custom_layout': 'mail.mail_notification_light',
        }
        return {
            'type': 'ir.actions.act_window',
            'view_mode': 'form',
            'res_model': 'mail.compose.message',
            'target': 'new',
            'context': ctx,
        }

    def print_repair_order(self):
        return self.env.ref('custom_repair.action_report_custom_repair_order').report_action(self)
        
    def action_open_barcode_inventory(self):
        # Obtener el registro de 'stock.barcodes.action' con el ID 4
        barcode_action = self.env['stock.barcodes.action'].browse(4)
        menu_view = self.env['ir.actions.act_window'].search([
            ('res_model', '=',  'wiz.stock.barcodes.read.picking'), 
            '|', ('name', '=', 'Acciones de código de barras'), ('name', '=', 'Barcodes actions')
        ], limit=1)
        menu_action = self.env['ir.ui.menu'].search([
            ('action', '=', 'ir.actions.act_window,' + str(menu_view.id))
        ], limit=1)

        # Asegúrate de que el contexto incluya 'default_repair_order_id'
        context_with_repair_order = self.env.context.copy()
        context_with_repair_order.update({
            'active_id': barcode_action.id,
            'active_ids': barcode_action.ids,
            'active_model':	'stock.barcodes.action',
            'context_display_menu':	True,
            'control_panel_hidden':	True,
            'default_repair_order_id': self.id,
            'form_view_initial_mode': 'edit',
            'inventory_mode': True,
            'params': {    
                'action': menu_view.id,
                'cids': 1,
                'menu_id': menu_action.id,
                'model': 'wiz.stock.barcodes.read.picking',
                'view_type': 'form'
            },

        })
        
        barcode_action = barcode_action.with_context(context_with_repair_order)
        # Llamar a 'custom_open_action' en el contexto modificado
        action = barcode_action.custom_open_action()
        
        return action
    

    def action_view_delivery(self):
        return self._get_action_view_picking(self.picking_ids)
    
    def _get_action_view_picking(self, pickings):
        '''
        This function returns an action that display existing delivery orders
        of given sales order ids. It can either be a in a list or in a form
        view, if there is only one delivery order to show.
        '''
        action = self.env["ir.actions.actions"]._for_xml_id("stock.action_picking_tree_all")

        if len(pickings) > 1:
            action['domain'] = [('id', 'in', pickings.ids)]
        elif pickings:
            form_view = [(self.env.ref('stock.view_picking_form').id, 'form')]
            if 'views' in action:
                action['views'] = form_view + [(state,view) for state,view in action['views'] if view != 'form']
            else:
                action['views'] = form_view
            action['res_id'] = pickings.id
        # Prepare the context.
        picking_id = pickings.filtered(lambda l: l.picking_type_id.code == 'outgoing')
        if picking_id:
            picking_id = picking_id[0]
        else:
            picking_id = pickings[0]
        action['context'] = dict(self._context, default_partner_id=self.partner_id.id, default_picking_type_id=picking_id.picking_type_id.id, default_origin=self.name, default_group_id=picking_id.group_id.id)
        return action
    
    def action_finish_repair(self):
        self.ensure_one()
        if self.is_operator and not self.is_chief_operator and self.state == 'repair' and self.user_id != self.env.user:
            raise UserError(_("No puede finalizar una tarea de una orden de reparación no asignada."))
        '''
        if 'T' in self.tag_ids.mapped('name'):
            raise UserError(_("No se puede finalizar la tarea de reparación porque se encuentra en torneria."))
        '''
        self.repair_finished_bool = True
        return


class CustomRepairLineInherit(models.Model):
    _inherit = 'repair.line'

    state = fields.Selection([
        ('draft', 'Quotation'),
        ('confirmed', 'Confirmed'),
        ('ready', 'Ready to Repair'),
        ('under_repair', 'Under Repair'),
        ('done', 'Repaired'),
        ('cancel', 'Cancelled'),
        ('order', 'Order'),
        ('analysis', 'Analysis'),
        ('repair', 'Repair'),
        ('verification', 'Verification'),
        ('2binvoiced', 'Billing')
        #('billing', 'Billing'),
        ], string='Status',
        copy=False, default='order', readonly=True, tracking=True
    )

    created_in_state = fields.Selection([
        ('draft', 'Quotation'),
        ('confirmed', 'Confirmed'),
        ('ready', 'Ready to Repair'),
        ('under_repair', 'Under Repair'),
        ('done', 'Repaired'),
        ('cancel', 'Cancelled'),
        ('order', 'Order'),
        ('analysis', 'Analysis'),
        ('repair', 'Repair'),
        ('verification', 'Verification'),
        ('2binvoiced', 'Billing')
        #('billing', 'Billing'),
        ], string='Status',
        copy=False, default='order', readonly=True, tracking=True
    )

    product_id = fields.Many2one(
        'product.product', 'Product', required=True, check_company=True,
        domain="[('type', 'in', ['product', 'consu']), '|', ('company_id', '=', company_id), ('company_id', '=', False)]"
    )

    def _default_line_location_id(self):
        location = self.env['stock.location'].search([('complete_name', '=', 'ART/Stock')], limit=1)
        return location.id if location else False

    location_id = fields.Many2one(
        'stock.location', 'Source Location',
        index=True, required=True,
        check_company=True,
        default=_default_line_location_id
    )

    @api.onchange('type')
    def onchange_operation_type(self):
        super(CustomRepairLineInherit, self).onchange_operation_type()
        self.location_id = self._default_line_location_id()

    def create(self, vals):
        if type(vals) is list:
            for val in vals:
                repair_order = self.env['repair.order'].search([
                    ('id', '=', val['repair_id'])
                ])
                if repair_order.state == 'done' or repair_order.state == '2binvoiced':
                    raise UserError(_('You cannot add a part to a completed repair order.'))
                val['created_in_state'] = repair_order.state
        return super(CustomRepairLineInherit, self).create(vals)

    def unlink(self):
        if self.repair_id.state == 'done' or self.repair_id.state == '2binvoiced':
            raise UserError(_('You cannot delete a part from a completed repair order.'))
        
        return super(CustomRepairLineInherit, self).unlink()
