import logging
from odoo import _, fields, models, api
from odoo.tools.safe_eval import safe_eval

_logger = logging.getLogger(__name__)


class CustomRepairStockBarcodesActionInherit(models.Model):
    _inherit = "stock.barcodes.action"
    
    def custom_open_action(self):
        action = self.action_window_id.sudo().read()[0]
        action_context = safe_eval(action["context"])
        ctx = self.env.context.copy()
        if action_context:
            ctx.update(action_context)
        if self.context:
            ctx.update(safe_eval(self.context))
        if action_context.get("inventory_mode", False):
            return self.custom_open_inventory_action(ctx)
        action["context"] = ctx
        return action

    def custom_open_inventory_action(self, ctx):
        option_group = self.env.ref(
            "stock_barcodes.stock_barcodes_option_group_inventory"
        )
        vals = {
            "option_group_id": option_group.id,
            "manual_entry": option_group.manual_entry,
        }
        if option_group.get_option_value("location_id", "filled_default"):
            vals["location_id"] = (
                self.env["stock.warehouse"].search([])[:1].lot_stock_id.id
            )
        wiz = self.env["wiz.stock.barcodes.read.inventory"].create(vals)
        action = self.env["ir.actions.actions"]._for_xml_id(
            "custom_repair.action_custom_repair_stock_barcodes_read_inventory"
        )
        action["res_id"] = wiz.id
        action["context"] = ctx
        return action
        
class CustomRepairWizStockBarcodesReadInventory(models.TransientModel):
    _inherit = "wiz.stock.barcodes.read.inventory"
    
    repair_order_id = fields.Many2one('repair.order', string='Repair Order')

    #update_view = fields.Boolean(default=False)

    def apply_inventory_to_repair_order(self):
        repair_order_id = self.repair_order_id.id
        for quant in self.inventory_quant_ids:
            self.env['repair.line'].create({
                'name': quant.product_id.display_name,
                'repair_id': self.repair_order_id.id,
                'product_id': quant.product_id.id,
                'price_unit': quant.product_id.list_price,
                'product_uom_qty': quant.inventory_quantity,
                'product_uom': quant.product_id.uom_id.id,
                'location_id': self.location_id.id,
                'location_dest_id': quant.product_id.property_stock_production.id,
                'state': self.repair_order_id.state,
            })

        for quant in self.inventory_quant_ids:
            quant.inventory_quantity = False
            quant.inventory_diff_quantity = False
            quant.inventory_quantity_set = False
        self.unlink()  # Eliminar el registro actual de wiz.stock.barcodes.read.inventory
          
        return {
            'type': 'ir.actions.act_window',
            'name': 'Repair Order',
            'res_model': 'repair.order',
            'view_mode': 'form',
            'res_id': repair_order_id,
            'target': 'main',
            'clear_breadcrumbs': True,  # Limpiar los breadcrumbs
        }
        

    def back_to_repair_order(self):
        repair_order_id = self.repair_order_id.id
        for quant in self.inventory_quant_ids:
            quant.inventory_quantity = False
            quant.inventory_diff_quantity = False
            quant.inventory_quantity_set = False
        self.unlink()  # Eliminar el registro actual de wiz.stock.barcodes.read.inventory
        
        return {
            'type': 'ir.actions.act_window',
            'name': 'Repair Order',
            'res_model': 'repair.order',
            'view_mode': 'form',
            'res_id': repair_order_id,
            'target': 'main',
            'clear_breadcrumbs': True,  # Limpiar los breadcrumbs
        }

    def action_done(self):
        result = super(
            CustomRepairWizStockBarcodesRead,
            self.with_context(_stock_barcodes_skip_read_log=True),
        ).action_done()
        if result:
            if self.repair_order_id:
                result = self._add_inventory_quant_repair()
                if result:
                    self.action_clean_values()
            else:
                result = self._add_inventory_quant()
                if result:
                    self.action_clean_values()
        return result
    

    def _add_inventory_quant_repair(self):
        StockQuant = self.env["stock.quant"]
        quant = StockQuant.search(self._inventory_quant_domain(), limit=1)
        quant = quant.with_context(inventory_mode=True)
        if quant:
            if self.product_id.tracking == "serial" and (
                quant.inventory_quantity > 0.0 or self.product_qty != 1
            ):
                self._serial_tracking_message_fail()
                return False
            if self.manual_entry:
                quant.inventory_quantity = self.product_qty
                quant.quantity = self.product_qty
            else:
                quant.inventory_quantity += 1
                quant.quantity += 1

            #self.update_view = True
        else:
            if self.product_id.tracking == "serial" and self.product_qty != 1:
                self._serial_tracking_message_fail()
                return False
            quant = StockQuant.with_context(inventory_mode=True).create(
                self._prepare_stock_quant_values()
            )
        self.inventory_product_qty = quant.quantity
        return True



class CustomRepairWizStockBarcodesRead(models.AbstractModel):
    _inherit = "wiz.stock.barcodes.read"

    ALLOWED_PRODUCT_TYPES = ["product", "consu"]


    def play_sounds(self, res):
        if not self.ids:
            _logger.error("No IDs found in self.ids")
            return

        channel = "stock_barcodes-{}".format(self.ids[0])
        sound_channel = "stock_barcodes_sound-{}".format(self.ids[0])
        sound = "ok" if res else "ko"

        self.env["bus.bus"]._sendone(channel, sound_channel, {"sound": sound})

    def process_barcode_product_id(self):
        domain = self._barcode_domain(self.barcode)
        product = self.env["product.product"].search(domain)
        if product:
            if len(product) > 1:
                self._set_messagge_info("more_match", _("More than one product found"))
                return False
            elif product.type not in self.ALLOWED_PRODUCT_TYPES:
                self._set_messagge_info(
                    "not_found", _("The product type is not allowed")
                )
                return False
            self.action_product_scaned_post(product)
            # Check if product is in inventory_quant_ids and increment quantity
            
            if (
                self.option_group_id.fill_fields_from_lot
                and self.location_id
                and self.product_id
            ):
                quant_domain = [
                    ("location_id", "=", self.location_id.id),
                    ("product_id", "=", product.id),
                ]
                if self.lot_id:
                    quant_domain.append(("lot_id", "=", self.lot_id.id))
                if self.package_id:
                    quant_domain.append(("package_id", "=", self.package_id.id))
                if self.owner_id:
                    quant_domain.append(("owner_id", "=", self.owner_id.id))
                quants = self.env["stock.quant"].search(quant_domain)
                if quants:
                    self.set_info_from_quants(quants)
            return True
        return False
